'' =================================================================================================
''
''   File....... jm_mcp4725_fast.spin  * * * W O R K   I N   P R O G R E S S * * *
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 07 JUN 2017
''
'' =================================================================================================

                                                              
con { io pins }                                                  
                                                                   
  RX1  = 31                                                     ' programming / terminal
  TX1  = 30                                                      
                                                                 
  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28                                                      
                                                                 

con

  WR_4725   = %1100_000_0                                       ' I2C device ID for MCP4725

  WR_DAC    = %0100_0000
  WR_DAC_EE = %0110_0000
                                                        
                                                           
obj                                                              
                                                                 
  i2c : "jm_i2c_fast"                                           ' essential i2c methods

                                                           
var                                                               
                                                                  
  long  devid
                                                           
                                                                 
pub start(addr)

'' Start MCP4725 device
'' -- creates device ID for this object
'' -- uses boot EEPROM pins
'' -- addr is device address for chip, %000 to %111
''    * A2 and A1 bits are hard-wired into chip; A0 is user-configurable

  startx(SCL1, SDA1, addr)


pub startx(sclpin, sdapin, addr)

'' Start MCP4725 device
'' -- creates device ID for this object
'' -- sclpin and sdapin are connections to I2C buss
'' -- addr is device address for chip, %000 to %111
''    * A2 and A1 bits are hard-wired into chip; A0 is user-configurable

  i2c.setupx(sclpin, sdapin, 400_000)                           ' connect to i2C
  devid := WR_4725 | ((addr & %111) << 1)                       ' setup slave ID byte


pub stop

  i2c.terminate                                                 ' free i2c bus

  
pub present

'' Pings device, returns true if found
  
  return i2c.present(devid)


pub reset

  i2c.start
  i2c.write($00)
  i2c.write($06) 
  i2c.stop


pub wake_up

  i2c.start
  i2c.write($00)
  i2c.write($09) 
  i2c.stop


pub write_fast(value, pdbits)

'' Writes value to DAC

  i2c.start
  i2c.write(devid)
  i2c.write(((pdbits & %11) << 4) | ((value >> 8) & $F))
  i2c.write(value & $FF)
  i2c.stop


pub write_dac(value, pdbits)

'' Writes value to DAC

  i2c.start
  i2c.write(devid)
  i2c.write(WR_DAC | ((pdbits & %11) << 1))
  i2c.write((value & $FFF) >> 4)
  i2c.write(value << 4) 
  i2c.stop


pub write_dac_ee(value, pdbits)

  i2c.start
  i2c.write(devid)
  i2c.write(WR_DAC_EE | ((pdbits & %11) << 1))
  i2c.write((value & $FFF) >> 4)
  i2c.write(value << 4) 
  i2c.stop


pub busy : check

'' Returns true if EE write process is busy

  i2c.start
  i2c.write(devid | 1)                                          ' read
  check := i2c.read(i2c#NAK)
  i2c.stop

  check := (check & $80) == 0


pub read_dac : value

  i2c.start
  i2c.write(devid | 1)
  i2c.read(i2c#ACK)                                             ' skip status
  value.byte[1] := i2c.read(i2c#ACK)
  value.byte[0] := i2c.read(i2c#ACK)
  i2c.stop

  value >>= 4                                                   ' z-align value


pub read_ee : value

  i2c.start
  i2c.write(devid | 1)
  repeat 3
    i2c.read(i2c#ACK)                                           ' skip status & dac bytes
  value.byte[1] := i2c.read(i2c#ACK)
  value.byte[0] := i2c.read(i2c#ACK)
  i2c.stop

  value >>= 4                                                   ' z-align value

  
dat { license }

  
{{

  Copyright (C) 2017 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}